# Application Checker
# General Templates and Functions Module (Templates.pm)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

package Common;
use strict;
require Exporter;

@Common::ISA = 'Exporter';
@Common::EXPORT = qw(
		$appchecker_ver %appchecker_branch $is_packager_rpm
		$cert_sys_host $cert_sys_base
		&guess_platform &guess_test_arch
		&autoflush_on &is_process_running
		&abs_path &read_file
		&esc_path &esc_cmdline &esc_url &unesc_url &esc_id &unesc_id &esc_html &esc_js
	);

system('which dpkg >/dev/null 2>&1');
our $is_packager_rpm = ($? ? 1 : 0);
$is_packager_rpm = 1 if (!-f '/etc/debian_version');

# Certification System
our $cert_sys_host = 'www.linuxfoundation.org';
our $cert_sys_base = '/lsb-cert/';

# Helps to determine platform name.
sub guess_platform($) {
	if ($_[0] =~ m/^(amd64|x86_64|x86-64)$/i) {
		return 'x86-64';
	}
	elsif ($_[0] =~ m/^(ia32|i[3-6]86|x86)$/i) {
		return 'x86';
	}
	elsif ($_[0] =~ m/^(ia64)$/i) {
		return 'IA64';
	}
	elsif ($_[0] =~ m/^(ppc|ppc32)$/i) {
		return 'PPC32';
	}
	elsif ($_[0] =~ m/^(ppc64)$/i) {
		return 'PPC64';
	}
	elsif ($_[0] =~ m/^(s390)$/i) {
		return 'S390';
	}
	elsif ($_[0] =~ m/^(s390x)$/i) {
		return 'S390X';
	}
	else {
		return 'unknown';
	}
}

# Determines the ELF file architecture
sub guess_test_arch($) {
	my ($file) = @_;
	return 'unknown' if ((!-f $file) or (-s $file < 20));
	if (open(FILE, $file)) {
		my $elfhdr;
		read(FILE, $elfhdr, 20);
		close(FILE);
		my @e_machine_bytes;
		# Store 2 bytes for e_machine in little-endian order
		if (substr($elfhdr, 5, 1) eq "\x02") {
			@e_machine_bytes = (substr($elfhdr, 19, 1), substr($elfhdr, 18, 1));    # Big-Endian
		}
		else {
			@e_machine_bytes = (substr($elfhdr, 18, 1), substr($elfhdr, 19, 1));    # Little-Endian
		}
		my $e_machine = ord($e_machine_bytes[0]) + 16 * ord($e_machine_bytes[1]);
		my $e_width = substr($elfhdr, 4, 1);            # 1: 32-bit; 2: 64-bit
		if (($e_machine == 0x03) and ($e_width eq "\x01")) {
			return 'x86';
		}
		elsif (($e_machine == 0x3e) and ($e_width eq "\x02")) {
			return 'x86-64';
		}
		elsif (($e_machine == 0x32) and ($e_width eq "\x02")) {
			return 'IA64';
		}
		elsif (($e_machine == 0x14) and ($e_width eq "\x01")) {
			return 'PPC32';
		}
		elsif (($e_machine == 0x15) and ($e_width eq "\x02")) {
			return 'PPC64';
		}
		elsif (($e_machine == 0x16) and ($e_width eq "\x01")) {
			return 'S390';
		}
		elsif (($e_machine == 0x16) and ($e_width eq "\x02")) {
			return 'S390X';
		}
		else {
			return 'unknown';
		}
	}
	else {
		return 'unknown';
	}
}

sub autoflush_on() {
	my $old_handle = select(STDOUT);
	$| = 1;
	select(STDERR);
	$| = 1;
	select($old_handle);
}

sub is_process_running($) {
	my ($PID) = @_;
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g;
	return (($test_line ne '') and ($test_line !~ m/^Z/i));
}

# Calculates absolute path to the specified entity.
# Unlike Cwd::abs_path, this variant can handle non-existent paths.
sub abs_path($) {
	my ($path) = @_;
	
	# General regexp tricks:
	# '<pattern>(?![^/])' means that the <pattern> is followed by a slash
	#                     or located at the end of line
	if ($path =~ m|^~|) {
		# First, expand tilde with home directories
		$path =~ s|^~([^/]+)(?![^/])|(getpwnam($1))[7]|e;
		$path =~ s|^~(?![^/])|$ENV{'HOME'}|;
	}
	elsif ($path !~ m|^/|) {
		# If the path does not start with '/', it is a relative path;
		# prepend it with the current directory
		my $cwd = `pwd`;
		$cwd =~ s/[\r\n]//g;
		$path = "$cwd/$path";
	}
	# Remove all repetitive slashs (replace each such sequence with single slash)
	$path =~ s|//+|/|g;
	# ...and the trailing slash
	$path =~ s|/$||;
	# Remove single-dot occurences
	$path =~ s|/\.(?![^/])||g;
	# Collapse sequences like '/directory/..'
	while ($path =~ s|/(?!\./)(?!\.\./)[^/]+/\.\.(?![^/])||g) {}
	# Remove excessive double-dot occurences applied to the root ('/../..')
	$path =~ s|^/(\.\./)*\.\.(?![^/])|/|;
	# If remained path is empty, make it root
	$path = '/' if ($path eq '');
	return $path;
}

# Reads the file contents and returns it (silently returns "" in case of error).
sub read_file($) {
	my ($filename) = @_;
	my $text = '';
	my $fh;

	if (open($fh, $filename)) {
		read($fh, $text, -s $fh);
		close($fh);
	}
	return $text;
}


# Escapes file system path so that it could be used in command line parameters.
# E.g.: /tmp/some path/dir(1)/a'readme.txt --> '/tmp/some path/dir(1)/a'\''readme.txt'
sub esc_path($) {
	my ($s) = @_;
	$s =~ s/\'/'\\\''/g;
	return "'$s'";
}

# Escape variable values for running the shell command.
# Values to be escaped should be enclosed in double angle quotation marks: «...»
# The file is to be opened in UTF-8 encoding, otherwise quotation marks may look different
# (which does not cause any problems if you copy-paste them exactly as they are, since
# processing is octet-based anyway).
# NO special Perl pragmas are needed!
# Usage sample: system(esc_cmdline("mv «$path1» «$path2/$newname»"));
sub esc_cmdline($) {
	my ($str) = @_;
	$str =~ s/«(.*?)»/esc_path($1)/eg;
	return $str;
}

# Escapes string so that it could be used in HTML URLs, GET/POST requests and cookies.
# According to RFC 2396 (2.3), the characters -_.!~*'() are allowed within URI components.
# But single quote we escape, to avoid problems with HTML and JS strings.
# Second parameter (optional) allows to exclude some characters from escaping.
# E.g.: This is a comment (#5) --> This+is+a+comment+(%235)
sub esc_url($;$) {
	my ($str, $params) = @_;
	my $excl = ($params->{'NOESC'} or '');
	$str =~ s/([^0-9a-zA-Z\-_.!~*() \Q$excl\E])/sprintf('%%%02X', ord($1))/eg;
	$str =~ s/ /\+/g;
	return $str;
}

# Unescapes the string encoded with esc_url().
# E.g.: This+is+a+comment+%28%235%29 --> This is a comment (#5)
sub unesc_url($) {
	my ($str) = @_;
	$str =~ s/\+/ /g;
	$str =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/eg;
	return $str;
}

# Escapes string so that it could be used in HTML IDs and Names.
# Quotation of the W3C Standard:
#   ID and NAME tokens must begin with a letter ([A-Za-z]) and may be followed by any number of letters,
#   digits ([0-9]), hyphens ("-"), underscores ("_"), colons (":"), and periods (".").
# Here we suppose that the resultant ID will begin with a letter, and do invalid character escaping only.
# Dots are used as escape-characters, so they themselves are escaped too.
# Colons are escaped as well, so that scripts could use them for their needs.
# Second parameter (optional) allows to exclude some characters from escaping.
# E.g.: File ID /tmp/1+1.txt --> File.20ID.20.2Ftmp.2F1.2B1.2Etxt
sub esc_id($;$) {
	my ($str, $params) = @_;
	my $excl = ($params->{'NOESC'} or '');
	$str =~ s/([^0-9a-zA-Z_\-\Q$excl\E])/sprintf('.%02X', ord($1))/eg;
	return $str;
}

# Unescapes the string encoded with esc_url().
# E.g.: File.20ID.20.2Ftmp.2F1.2B1.2Etxt --> File ID /tmp/1+1.txt
sub unesc_id($) {
	my ($str) = @_;
	$str =~ s/\.([0-9a-fA-F]{2})/chr(hex($1))/eg;
	return $str;
}

# Escapes string, replacing special HTML characters (plus quotation characters) with HTML entities.
# E.g.: Text & <b>numbers</b> --> Text &amp; &lt;b&gt;numbers&lt;/b&gt;
sub esc_html($) {
	my ($str) = @_;
	$str =~ s/&/&amp;/g;
	$str =~ s/</&lt;/g;
	$str =~ s/>/&gt;/g;
	$str =~ s/\"/&quot;/g;
	return $str;
}

# Escapes quotation characters in string, so that it could be used in JavaScript strings.
# E.g.: /tmp/dir(1)/a'readme["test"].txt --> /tmp/dir(1)/a\x27readme[\x22test\x22].txt
sub esc_js($) {
	my ($str) = @_;
	$str =~ s/\'/\\x27/g;
	$str =~ s/\"/\\x22/g;
	return $str;
}

# Initialize AppChecker version and branch
our $appchecker_ver = 'undefined';
our %appchecker_branch;
my $ver_path = abs_path(__FILE__);
$ver_path =~ s!^(.*/)[^/]+$!$1VERSION!;
my $ver_line = read_file($ver_path);
$appchecker_branch{'STD'} = 'LSB';
if ($ver_line =~ m/^([^:]*):(.*)$/) {
	$appchecker_ver = $1;
	$appchecker_branch{'STD'} = $2;
}
$appchecker_branch{'OS'} = (($appchecker_branch{'STD'} eq 'LSB') ? 'Linux' : $appchecker_branch{'STD'});
$appchecker_branch{'DIR'} = lc($appchecker_branch{'STD'});
$appchecker_branch{'USER'} = lc($appchecker_branch{'OS'}).'appchecker';
$appchecker_branch{'NAVIGATOR'} = (($appchecker_branch{'STD'} eq 'LSB') ? 'http://dev.linuxfoundation.org/navigator/browse/' : 'http://dev.linuxfoundation.org/moblin-navigator/browse/');


1;
