#!/usr/bin/perl -w

# Application Checker
# AJAX Server Module (ajax_srv.pl)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Fcntl ':flock';
use Fcntl ':seek';
use File::Temp qw/tempfile tmpnam/;
use Templates;
use Update 'PROXY' => $SERVER_PARAM{'PROXY'}, 'ARCH' => lc($SERVER_PARAM{'ARCHITECTURE'});

my $error_text = '';
my $data = '';

my $output_xml = "HTTP/1.0 200 OK" . CRLF . "Content-type: text/xml" . CRLF . CRLF . "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";

sub send_reply() {
	if ($error_text) {
		$error_text = esc_html("<ul><li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$error_text</font></li></ul>");
		print "$output_xml<root>\n<error>$error_text</error>\n</root>";
	}
	else {
		print "$output_xml<root>\n$data</root>";
	}
}

sub list_dir() {
	if (opendir(DIR, $SERVER_PARAM{'APP_DATA'}.'/profiles')) {
		$data .= "<profiles>\n";
		foreach (sort(grep(!/^[~\.]/, readdir(DIR)))) {
			$data .= "<name>$_</name>\n";
		}
		$data .= "</profiles>\n";
		closedir(DIR);
	}
	else {
		$error_text = "Could not obtain directory list!<br />($!)";
	}
}

my $default_spec_version = (($appchecker_branch{'STD'} eq 'LSB') ? '4.1' : '2.0');

if (!$_GET{'action'}) {
	$error_text = 'Incorrect call parameters!';
}
elsif ($_GET{'action'} eq 'delete_profile') {               # Delete profile
	if ($_GET{'profile'}) {
		if (!-f $SERVER_PARAM{'APP_DATA'}.'/profiles/'.$_GET{'profile'}) {
			$error_text = 'File not found:<br />'.$SERVER_PARAM{'APP_DATA'}.'/profiles/'.$_GET{'profile'};
		}
		else {
			unlink($SERVER_PARAM{'APP_DATA'}.'/profiles/'.$_GET{'profile'});
			list_dir();
		}
	}
	else {
		$error_text = 'No profile name specified!';
	}
}
elsif ($_GET{'action'} eq 'save_profile') {                 # Save profile
	if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
		system(esc_cmdline("mkdir -p «$SERVER_PARAM{'APP_DATA'}/profiles»"));
		if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
			$error_text = 'Could not create profile directory:<br />'.$SERVER_PARAM{'APP_DATA'}.'/profiles';
		}
	}
	if (!$error_text) {
		if (!$_POST{'edit_profile_name'}) {
			$error_text = 'No profile name specified!';
		}
		elsif ($_POST{'edit_profile_name'} =~ m/[\\\/<>\|\?\*]/) {
			$error_text = 'Invalid profile name: '.$_POST{'edit_profile_name'};
		}
	}
	if (!$error_text) {
		if ($_POST{'application'} and ($_POST{'application'} =~ m/[\\\/<>\|\?\*]/)) {
			$error_text = 'Invalid application name: '.$_POST{'application'};
		}
	}
	if (!$error_text) {
		my $full_save_name = $SERVER_PARAM{'APP_DATA'}.'/profiles/'.$_POST{'edit_profile_name'};
		if (open(FILE, ">$full_save_name")) {
			my $architecture = ($_POST{'architecture'} or guess_test_arch($CONFIG{'TESTS_DIR'}.'/'.lc($appchecker_branch{'STD'}).'appchk'));
			my $spec_version = ($_POST{'spec_version'} or $default_spec_version);
			print FILE 'APPLICATION:     '.$_POST{'application'}."\n" if ($_POST{'application'});
			if ($_POST{'components'}) {
				foreach (split(/\r?\n/, $_POST{'components'})) {
					print FILE "COMPONENT:       $_\n" if ($_);
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "SPEC_VERSION:    $spec_version\n";
			if ($appchecker_branch{'STD'} eq 'LSB') {
				print FILE 'SPEC_PROFILE:    '.($_POST{'spec_profile'} or 'core,c++,desktop')."\n";
			}
			print FILE 'SHOW_HIDDEN:     '.$_POST{'show_hidden_files'}."\n" if (defined($_POST{'show_hidden_files'}));
			print FILE 'LAST_DIR:        '.$_POST{'last_opened_dir'}."\n" if ($_POST{'last_opened_dir'});

			if ($_POST{'comments'}) {
				my @comments_lines = split(/\r?\n/, $_POST{'comments'});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}
			close(FILE);
			list_dir();
		}
		else {
			$error_text = "Cannot open file for writing ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{'action'} eq 'load_profile') {                 # Load profile
	my %profile_data = (
		'application' => '',
		'components' => '',
		'architecture' => '',
		'spec_version' => $default_spec_version,
		'spec_profile' => 'core,c++,desktop',
		'show_hidden_files' => '0',
		'last_opened_dir' => '/',
		'comments' => ''
	);
	if (!$_GET{'profile'}) {
		$error_text = 'No profile name specified!';
	}
	if (!$error_text) {
		my $full_save_name = $SERVER_PARAM{'APP_DATA'}.'/profiles/'.$_GET{'profile'};
		if (open(FILE, $full_save_name)) {
			my @comments = ();
			my @components = ();
			while (<FILE>) {
				s/[\r\n]//g;
				if (/^APPLICATION:\s+(.*)$/) {
					$profile_data{'application'} = $1;
				}
				elsif (/^COMPONENT:\s+(.*)$/) {
					push @components, $1;
				}
				elsif (/^SPEC_VERSION:\s+(.*)$/) {
					$profile_data{'spec_version'} = $1;
				}
				elsif (/^SPEC_PROFILE:\s+(.*)$/ and ($appchecker_branch{'STD'} eq 'LSB')) {
					$profile_data{'spec_profile'} = $1;
				}
				elsif (/^ARCHITECTURE:\s+(.*)$/) {
					$profile_data{'architecture'} = guess_platform($1);
				}
				elsif (/^SHOW_HIDDEN:\s+(.*)$/) {
					$profile_data{'show_hidden_files'} = $1;
				}
				elsif (/^LAST_DIR:\s+(.*)$/) {
					$profile_data{'last_opened_dir'} = $1;
				}
				elsif (/^COMMENTS: (.*)$/) {
					push @comments, $1;
				}
			}
			close(FILE);
			$profile_data{'components'} = join("\n", @components);
			$profile_data{'comments'}   = join("\n", @comments);
			$data .= "<profile_data>\n";
			foreach (keys %profile_data) {
				$profile_data{$_} =~ s/\]\]>/]]>]]<![CDATA[>/g;
				$data .= "<item><name>$_</name><value><![CDATA[$profile_data{$_}]]></value></item>\n";
			}
			$data .= "</profile_data>\n";
			list_dir();
		}
		else {
			$error_text = "Cannot open file for reading ($!):<br />$full_save_name";
		}
	}
}
elsif ($_GET{'action'} eq 'stop_tests') {                   # Stop the tests
	if (!$_GET{'tree'} or ($_GET{'tree'} ne 'server')) {
		$error_text = 'Incorrect call parameters!';
	}
	else {
		if (open(SRVLOG, $SERVER_PARAM{'APP_DATA'}.'/log/app-checker-server.log.'.$SERVER_PARAM{'PORT'})) {
			my $PID = <SRVLOG>;
			close(SRVLOG);
			if ($PID =~ m/PID:\s*(\d+)/i) {
				$PID = $1;
				my $out_file = tmpnam();
				system(esc_cmdline("«$CONFIG{'UTILS_DIR'}/stop_tests.pl» $PID «$SERVER_PARAM{'APP_DATA'}/log/app-checker-server.log.$SERVER_PARAM{'PORT'}» >«$out_file» 2>&1"));
				if ($? == 0) {
					$data = "<stopped id=\"".$_GET{"tree"}."\">1</stopped>\n";
				}
				else {
					$error_text = read_file($out_file);
					$error_text =~ s/\n/<br \/>/g;
				}
				unlink($out_file);
			}
			else {
				$error_text = "Failed to extract the server PID from the log file.";
			}
		}
		else {
			$error_text = "Failed to open file '$SERVER_PARAM{'APP_DATA'}/log/app-checker-server.log.$SERVER_PARAM{'PORT'}' for reading:<br />$!";
		}
	}
}
elsif ($_GET{'action'} eq 'read_packages') {                # Get the list of the installed packages
	my $pkg_err_file = tmpnam();
	if ($is_packager_rpm) {
		my @packages = sort split(/\n/, `rpm -qa --qf "\%\{NAME\}\\t\%\{VERSION\}\\t\%\{RELEASE\}\\n" 2>$pkg_err_file`);
		if ($? != 0) {
			$error_text = 'Could not retrieve the list of installed packages:<br />'.read_file($pkg_err_file);
		}
		else {
			foreach my $pkg (@packages) {
				my ($name, $ver, $rel) = split(/\t/, $pkg, 3);
				# Skip GPG/DSA public keys stored in RPM database
				next if ($name eq 'gpg-pubkey');
				my $name_disp = esc_html($name);
				my $ver_disp  = esc_html($ver);
				my $rel_disp  = esc_html($rel);
				my $name_id = esc_id($name);
				my $ver_id  = esc_id($ver);
				my $rel_id  = esc_id($rel);
				my $pkg_id = $name_id.'::'.$ver_id.'::'.$rel_id;
				$data .= "<pkg><pkgid>$pkg_id</pkgid><nameid>$name_id</nameid><ufname>$name-$ver-$rel</ufname><namedisp>$name_disp</namedisp><verdisp>$ver_disp</verdisp><reldisp>$rel_disp</reldisp></pkg>\n";
			}
		}
	}
	else {
		my @packages = sort split(/\n/, `dpkg-query -W -f '\${Package}\t\${Version}\t\${Status}\n' 2>$pkg_err_file`);
		if ($? != 0) {
			$error_text = 'Could not retrieve the list of installed packages:<br />'.read_file($pkg_err_file);
		}
		else {
			foreach my $pkg (@packages) {
				my ($name, $ver, $st) = split(/\t/, $pkg, 3);
				# Skip non-installed packages
				next if ($st ne 'install ok installed');
				my $name_disp = esc_html($name);
				my $ver_disp  = esc_html($ver);
				my $name_id = esc_id($name);
				my $ver_id  = esc_id($ver);
				my $pkg_id = $name_id.'::'.$ver_id;
				$data .= "<pkg><pkgid>$pkg_id</pkgid><nameid>$name_id</nameid><ufname>${name}_${ver}</ufname><namedisp>$name_disp</namedisp><verdisp>$ver_disp</verdisp></pkg>\n";
			}
		}
	}
	unlink($pkg_err_file);
}
elsif ($_GET{'action'} eq 'check_db_updates') {             # Check for DB snapshots update
	my ($upd_ver, $dl_size) = db_check_update($CONFIG{'DBDATA_DIR'});
	if (!defined($upd_ver)) {
		$error_text = join('<br />', @db_error_msg);
	}
	else {
		if ($upd_ver eq '') {
			$data .= '<dbver>0</dbver>';
		}
		else {
			$data .= '<dbver>'.$upd_ver.'</dbver><dbverfmt>'.date_format($upd_ver).'</dbverfmt>';
		}
		my ($dbver_current, $checked_date) = db_get_version($CONFIG{'DBDATA_DIR'});
		if (!$dbver_current) {
			$dbver_current = 'undefined';
		}
		if (!$checked_date) {
			$checked_date = 'never';
		}
		$data .= '<curdbver>'.date_format($dbver_current).'</curdbver><checked>'.date_format($checked_date).'</checked>';
	}
}
elsif ($_GET{'action'} eq 'do_db_updates') {                # Performs DB snapshots update
	my $lock_file = $CONFIG{'DBDATA_DIR'}.'/db_update.lock';
	my $info_file = $CONFIG{'DBDATA_DIR'}.'/db_update.info';
	# Create/open the lock file
	if (open(LOCK_FILE, '>'.$lock_file)) {
		# If we can lock it exclusively, then no other updating process will interfere
		if (flock(LOCK_FILE, LOCK_EX | LOCK_NB)) {
			my $child = fork();
			if (!defined($child)) {
				unlink($lock_file);
				flock(LOCK_FILE, LOCK_UN);
				close(LOCK_FILE);
				$error_text = 'Internal error: fork failed!';
			}
			elsif ($child == 0) {
				# Open the info file to store current progress
				if (open(INFO_FILE, '+>'.$info_file)) {
					# Turn auto-flush on
					my $oldh = select(INFO_FILE);
					$| = 1;
					select($oldh);
					# Close STDOUT so that the server could finish transmission
					close(STDOUT);
					# Reopen STDOUT, so that opening other file handles did not take its id
					# (known problem of Perl)
					open(STDOUT, '>/dev/null');

					# Callback function to store the current progress
					my $update_callback = sub($$$) {
						my ($total_percent, $file_name, $file_percent) = @_;
						my $str = "$total_percent $file_percent $file_name";
						flock(INFO_FILE, LOCK_EX);
						seek(INFO_FILE, 0, SEEK_SET);
						print INFO_FILE $str;
						truncate(INFO_FILE, length($str));
						flock(INFO_FILE, LOCK_UN);
					};

					# Start updating
					my $upd_ver = db_update($CONFIG{'DBDATA_DIR'}, \&$update_callback);
					close(INFO_FILE);
					unlink($info_file);
				}
				else {
					# Here STDOUT is not closed, so the server is still reading the
					# output and waiting for this child process to finish, so we can
					# write a message into STDERR and it will come to the server log.
					# We cannot send error to STDOUT: it would mix with with the XML
					# output of the parent process.
					print STDERR "Failed to open info file: $info_file<br />$!";
				}
				# Unlock and remove the lock file
				unlink($lock_file);
				flock(LOCK_FILE, LOCK_UN);
				close(LOCK_FILE);
				# Do not need to send reply, silently exit
				exit;
			}
			else {
				close(LOCK_FILE);
				for (my $i = 0; $i < 3; ++$i) {
					select(undef, undef, undef, 0.3);
					last if (-f $info_file);
				}
				$data .= '<updating>1</updating>';
			}
		}
		else {
			close(LOCK_FILE);
			$error_text = 'Updating is in progress already!';
		}
	}
	else {
		$error_text = "Failed to open lock file: $lock_file<br />$!";
	}
}
elsif ($_GET{'action'} eq 'check_db_update_status') {       # Returns current status of DB update process
	my $info_file = $CONFIG{'DBDATA_DIR'}.'/db_update.info';
	if (open(INFO_FILE, $info_file)) {
		flock(INFO_FILE, LOCK_EX);
		my $info = <INFO_FILE>;
		if ($info) {
			$info =~ s/[\r\n]//g;
			my @info_parts = split(/\s+/, $info, 3);
			$data .= "<total>$info_parts[0]</total><file>$info_parts[1]</file><filename>$info_parts[2]</filename>";
		}
		else {
			$data .= '<total>0</total><file>0</file><filename> </filename>';
		}
		flock(INFO_FILE, LOCK_UN);
		close(INFO_FILE);
	}
	else {
		$data .= '<total>100</total>';
	}
	my ($dbver_current, $checked_date) = db_get_version($CONFIG{'DBDATA_DIR'});
	if (!$dbver_current) {
		$dbver_current = 'undefined';
	}
	if (!$checked_date) {
		$checked_date = 'never';
	}
	$data .= '<curdbver>'.date_format($dbver_current).'</curdbver><checked>'.date_format($checked_date).'</checked>';
}
elsif ($_GET{'action'} eq 'server_control') {               # Updates the config file and forces the server to re-read it
	if (!defined($_GET{'proxy'})) {
		$error_text = 'Internal error: Proxy server field was not passed.';
	}
	else {
		if (open(CONF_FILE, $SERVER_PARAM{'CONF_FILE'})) {
			flock(CONF_FILE, LOCK_EX);
			my $conf_file_new = $SERVER_PARAM{'CONF_FILE'};
			$conf_file_new =~ s!/[^/]+$!/!;
			(my $fh_new, $conf_file_new) = tempfile(DIR => $conf_file_new);
			my $saved = 0;
			while (my $line = <CONF_FILE>) {
				if ($line =~ m/^\s*ProxyServer\b/) {
					if (!$saved) {
						print $fh_new "ProxyServer = $_GET{'proxy'}\n";
						$saved = 1;
					}
				}
				else {
					print $fh_new $line;
				}
			}
			if (!$saved) {
				print $fh_new "\nProxyServer = $_GET{'proxy'}\n";
			}
			close($fh_new);
			if (!unlink($SERVER_PARAM{'CONF_FILE'})) {
				$error_text = "Failed to unlink the old file $SERVER_PARAM{'CONF_FILE'}:<br />$!";
			}
			else {
				if (!rename($conf_file_new, $SERVER_PARAM{'CONF_FILE'})) {
					$error_text = "Failed to rename the new file $conf_file_new to $SERVER_PARAM{'CONF_FILE'}:<br />$!";
				}
			}
			flock(CONF_FILE, LOCK_UN);
			close(CONF_FILE);
		}
		else {
			my $conf_file_new = $SERVER_PARAM{'CONF_FILE'};
			$conf_file_new =~ s!/[^/]+$!/!;
			(my $fh_new, $conf_file_new) = tempfile(DIR => $conf_file_new);
			print $fh_new "ProxyServer = $_GET{'proxy'}\n";
			close($fh_new);
			if (!rename($conf_file_new, $SERVER_PARAM{'CONF_FILE'})) {
				$error_text = "Failed to rename the new file $conf_file_new to $SERVER_PARAM{'CONF_FILE'}:<br />$!";
			}
		}
		if (!$error_text) {
			if (kill('HUP', $SERVER_PARAM{'SERVER_PID'})) {
				$data .= '<serverupd>1</serverupd>';
			}
			else {
				$error_text = "Failed to send SIGHUP to the server:<br />$!";
			}
		}
	}
}
elsif ($_GET{'action'} eq 'results_table') { # Applies selected filter and returns a new filtered table ('Libraries' and 'Interfaces' tabs)
	{ # a block to be able to do last()
		my $table = $_GET{'table'};
		if (!$table || $table ne 'libs' && $table ne 'ints') {
			$error_text = "Unsupported 'table' value: '$table'";
			last;
		}

		my $details = $_GET{'details'};
		if (!$details) {
			$error_text = "Unsupported 'table' value: '$table'";
			last;
		}

		if (!$CONFIG{'RESULTS_DIR'}) {
			$error_text = 'Internal error: RESULTS_DIR undefined.';
			last;
		}

		use Tests_results;

		# Load the analysis data
		my $res_dir = $CONFIG{'RESULTS_DIR'}.'/'.$details;

		my ($app_name, $host_platform, $spec_version) = read_results_info($res_dir);

		my $err = load_analysis_data($res_dir);
		if ($err) {
			$error_text = $err;
			last;
		}

		if ($table eq 'libs') {
			my $distr_sel = Tests_results::get_distr_sel();
			my $filt_com = $_COOKIE{'lfc'};
			my $filt_show = $_COOKIE{'lfs'};

			my $filtered = $filt_com || $filt_show || scalar grep {!$_} values %$distr_sel;

			# Prepare data
			my ($libs) = Tests_results::libs_data($distr_sel, $filt_com);

			my ($html_table, $row_n) = Tests_results::L_table($libs, $spec_version, $distr_sel,
			                                                  $filt_com, $filt_show);

			# Make the response
			$data .= '<xml>';
			if ($filtered) {
				$data .= '<row_n>'.$row_n.'</row_n>';
			}
			$html_table =~ s(\]\]>)(]]>]]<![CDATA[>)g;  # Escape XML CDATA's ending tag
			$data .= '<l_table><![CDATA['.$html_table.']]></l_table>';
			$data .= '</xml>';
		}
		elsif ($table eq 'ints') {
			my $distr_sel = Tests_results::get_distr_sel();
			my $filt_com = $_COOKIE{'ifc'};
			my $filt_lib = $_COOKIE{'ifl'};
			my $filt_show = $_COOKIE{'ifs'};

			my $filtered = $filt_com || $filt_lib || $filt_show || scalar grep {!$_} values %$distr_sel;

			# Prepare data
			my ($ints) = Tests_results::ints_data($distr_sel, $filt_com, $filt_lib);

			my ($html_table, $row_n) = Tests_results::I_table($ints, $spec_version, $distr_sel,
			                                                  $filt_com, $filt_lib, $filt_show);

			# Make the response
			$data .= '<xml>';
			if ($filtered) {
				$data .= '<row_n>'.$row_n.'</row_n>';
			}
			$html_table =~ s(\]\]>)(]]>]]<![CDATA[>)g;  # Escape XML CDATA's ending tag
			$data .= '<i_table><![CDATA['.$html_table.']]></i_table>';
			$data .= '</xml>';
		}
	}
}
else {
	$error_text = 'Incorrect call parameters!';
}

send_reply();
