#!/usr/bin/perl -w

# Application Checker
# Configuration Save Module (tests_conf_save.pl)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';
use Fcntl ':seek';
use Cwd 'cwd';

autoflush_on();

my $conf_page = 'tests_conf.pl';
if ($_GET{'goto'}) {
	$conf_page = $_GET{'goto'};
	$conf_page =~ s/%23/#/;
	$_POST{'form_action'} = '';
}
my $session_profile = ($_COOKIE{'session_id'} ? '~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$_COOKIE{'session_id'} : '');
if (($session_profile eq '') and ($_POST{'session_id'})) {
	$session_profile = '~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$_POST{'session_id'};
}

my $error;

if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
	system(esc_cmdline("mkdir -p «$SERVER_PARAM{'APP_DATA'}/profiles»"));
	if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
		$error = '';
	}
}

if (!defined($error)) {
	if ($_POST{'form_action'}) {
		if ($_POST{'form_action'} eq 'run_tests') {
			if ($session_profile eq '') {
				$error = 'Cannot retrieve session ID! Please allow cookies in your browser.';
			}
		}
		else {
			$error = 'Unknown script action specified!';
		}
	}
	elsif (!$_GET{'goto'}) {
		$error = 'Script action not specified!';
	}
}

if (!defined($error)) {
	if ($session_profile ne '') {
		my $full_save_name = $SERVER_PARAM{'APP_DATA'}."/profiles/$session_profile";
		if (open(FILE, "> $full_save_name")) {
			my $architecture = ($_POST{'architecture'} or guess_test_arch($CONFIG{'TESTS_DIR'}.'/'.lc($appchecker_branch{'STD'}).'appchk'));
			my $spec_version = ($_POST{'spec_version'} or (($appchecker_branch{'STD'} eq 'LSB') ? '4.1' : '2.0'));
			my $spec_profile = ($_POST{'spec_profile'} or 'core,c++,desktop');
			print FILE 'APPLICATION:     '.$_POST{'application'}."\n" if ($_POST{'application'});
			if ($_POST{'components'}) {
				$_POST{'components'} =~ s/\r\n/\n/g;
				foreach (split(/\n/, $_POST{'components'})) {
					s/^\s*(.*?)\s*$/$1/;
					print FILE "COMPONENT:       $_\n" if ($_);
				}
			}
			print FILE "ARCHITECTURE:    $architecture\n";
			print FILE "SPEC_VERSION:    $spec_version\n";
			print FILE "SPEC_PROFILE:    $spec_profile\n" if ($appchecker_branch{'STD'} eq 'LSB');
			print FILE 'SHOW_HIDDEN:     '.$_POST{'show_hidden_files'}."\n" if (defined($_POST{'show_hidden_files'}));
			print FILE 'LAST_DIR:        '.$_POST{'last_opened_dir'}."\n" if ($_POST{'last_opened_dir'});

			if ($_POST{'comments'}) {
				my @comments_lines = split(/\r?\n/, $_POST{'comments'});
				foreach (@comments_lines) {
					print FILE "COMMENTS: $_\n";
				}
			}

			close(FILE);

			if ($_POST{'form_action'} eq 'run_tests') {
				if (!$_POST{'application'}) {
					$error = 'Application name is not specified.';
				}
				else {
					print 'HTTP/1.0 200 OK' . CRLF;
					print 'Content-type: text/html' . CRLF . CRLF;
					print_header('Test Execution', 'conf');

					# List of progress table elements
					my @progress_elems = (
						{'ID' => 'check-pkg',     'DESC' => 'Checking packages',                       'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'check-shell',   'DESC' => 'Checking Shell scripts',                  'LSB' => 1, 'Moblin' => 0},
						{'ID' => 'check-perl',    'DESC' => 'Checking Perl scripts',                   'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'check-python',  'DESC' => 'Checking Python scripts',                 'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'check-elf-so1', 'DESC' => 'Checking ELF shared objects: pass1',      'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'check-elf-so2', 'DESC' => 'Checking ELF shared objects: pass2',      'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'check-elf-bin', 'DESC' => 'Checking ELF binaries',                   'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'check-desktop', 'DESC' => 'Checking .desktop files',                 'LSB' => 0, 'Moblin' => 1},
						{'ID' => 'report-gen',    'DESC' => 'Processing test journals',                'LSB' => 1, 'Moblin' => 1},
						{'ID' => 'app-data',      'DESC' => 'Collecting data about application files', 'LSB' => 1, 'Moblin' => 1}
					);
					my $elems_js_list = join(', ', map("'$_->{'ID'}'", @progress_elems));

					# Show timer with waiting message
					print <<DATA;
<div id="progress_info">
<font color="green"><b>Please wait while the test is running...</b></font><br />
Time elapsed: <span id="timer">00:00</span>
</div>

<script language="javascript" type="text/javascript">
// <![CDATA[
var elapsed_time = 0;
var finished = 0;
function update_timer() {
	++elapsed_time;
	var elem = document.getElementById('timer');
	if (elem && !finished) {
		var m = Math.floor(elapsed_time / 60);
		var s = elapsed_time - m * 60;
		var tmp = '';
		tmp += (m < 10) ? '0' + m : m;
		tmp += ':';
		tmp += (s < 10) ? '0' + s : s;
		elem.innerHTML = tmp;
		setTimeout('update_timer()', 1000);
	}
}

setTimeout('update_timer()', 1000);

var ids = new Array($elems_js_list);

var elem_num = new Array();
var elem_total = new Array();
var elem_file = new Array();
for (i in ids) {
	var id = ids[i];
	elem_num[id] = 0;
	elem_total[id] = 0;
	elem_file[id] = '';
}

var progress_current = 'file-list';

function update_elems_repeat() {
	update_elems();
	if (!finished)
		setTimeout('update_elems_repeat()', 300);
}

function update_elems() {
	var elem;
	for (var i in ids) {
		var id = ids[i];
		elem = document.getElementById(id);
		if (elem)
			elem.style.color = ((elem_total[id] == 0) ? 'gray' : 'black');
		elem  = document.getElementById(ids[i] + '-num');
		if (elem)
			elem.innerHTML = elem_num[id] + '/' + elem_total[id];
		elem = document.getElementById(ids[i] + '-elem');
		if (elem)
			elem.innerHTML = elem_file[id];
	}
}

setTimeout('update_elems_repeat()', 300);

function changeProgressElem(id) {
	var elem;
	if (progress_current && (elem_total[progress_current] !== 0)) {
		elem = document.getElementById(progress_current + '-img');
		if (elem) {
			elem.setAttribute('src', 'images/exec_done.gif');
			elem.setAttribute('alt', 'Done');
		}
	}
	progress_current = id;
	if (progress_current && (elem_total[progress_current] !== 0)) {
		elem = document.getElementById(progress_current + '-img');
		if (elem) {
			elem.setAttribute('src', 'themes/$appchecker_branch{'DIR'}/images/exec_progress.gif');
			elem.setAttribute('alt', 'Running');
		}
	}
}

// ]]>
</script>

<br />

<table class="wikitable" cellpadding="2" cellspacing="0">
  <tr>
    <th></th>
    <th style="width: 25em;">Stage</th>
    <th style="width: 30em;">Current file</th>
  </tr>
  <tr>
    <td><img id="file-list-img" src="themes/$appchecker_branch{'DIR'}/images/exec_progress.gif" alt="Running" width="16" height="16" /></td>
    <td>Creating list of files</td>
    <td></td>
  </tr>
DATA

					for my $progress_elem (@progress_elems) {
						next if (!$progress_elem->{$appchecker_branch{'STD'}});
						my $id = $progress_elem->{'ID'};
						print <<DATA;
  <tr>
    <td><img id="$id-img" src="images/environment-spacer.gif" alt="Not started" width="16" height="16" /></td>
    <td id="$id" style="color: gray;">$progress_elem->{'DESC'} (<span id="$id-num">0/0</span>)</td>
    <td id="$id-elem" style="white-space: nowrap;"></td>
  </tr>
DATA
					}
					print <<DATA;
  <tr>
    <td><img id="merge-img" src="images/environment-spacer.gif" alt="Not started" width="16" height="16" /></td>
    <td id="merge">Analysing collected data</td>
    <td></td>
  </tr>
  <tr>
    <td><img id="cleanup-img" src="images/environment-spacer.gif" alt="Not started" width="16" height="16" /></td>
    <td id="cleanup">Removing temporary files</td>
    <td></td>
  </tr>
</table>
DATA
					print_footer(1);

					my $result_dir;
					my %progress_elems = (
						'PKG' => 'check-pkg',
						'Shell' => 'check-shell',
						'Perl' => 'check-perl',
						'Python' => 'check-python',
						'ELF-so1' => 'check-elf-so1',
						'ELF-so2' => 'check-elf-so2',
						'ELF-bin' => 'check-elf-bin',
						'DESKTOP' => 'check-desktop',
						'JOURNAL' => 'report-gen',
						'APPDATA' => 'app-data',
						'MERGE' => 'merge',
						'CLEANUP' => 'cleanup',
						'FINISH' => ''
					);
					my $res = 0;
					if (open(TEST_RUN, esc_cmdline("«$CONFIG{'UTILS_DIR'}/run_tests.pl» --verbose --profile=«$full_save_name» --results-dir=«$CONFIG{'RESULTS_DIR'}» --tests-dir=«$CONFIG{'TESTS_DIR'}» --dbdata-dir=«$CONFIG{'DBDATA_DIR'}» |"))) {
						my $line;
						my $current = '';
						while ($line = <TEST_RUN>) {
							$line =~ s/[\r\n]//g;
							if ($line =~ m/^PROGRESS: (.*)$/) {
								my $info = $1;
								my $js_code = '';
								if ($info =~ m/Add: (\S+)\.$/) {
									$js_code = "++elem_total['$progress_elems{$1}'];";
								}
								elsif ($info =~ m/Add-processed: (\S+)\.$/) {
									$js_code = "++elem_total['$progress_elems{$1}'];\n++elem_num['$progress_elems{$1}'];";
								}
								elsif ($info =~ m/Process: (\S+) \((.*)\)\.$/) {
									my ($elem, $file) = ($1, $2);
									$file = esc_js(esc_html($file));
									$js_code = "++elem_num['$progress_elems{$elem}'];\nelem_file['$progress_elems{$elem}'] = '$file';";
								}
								elsif ($info =~ m/^Starting (\S+).$/) {
									$js_code = ($current ? "elem_file['$current'] = '';\n" : '')."update_elems();\nchangeProgressElem('$progress_elems{$1}');";
									$current = $progress_elems{$1};
								}
								if ($js_code) {
									print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
$js_code
// ]]>
</script>
DATA
								}
							}
							elsif ($line =~ m/^ERROR: (.*)$/i) {
								$error .= ($error ? '<br />' : '').$1;
							}
							if ($line =~ m/^RESULT: (.*)$/i) {
								$result_dir = $1;
							}
						}
						close(TEST_RUN);
						$res = $?;
					}
					else {
						$error = "Failed to start run_tests.pl:<br />$!";
					}

					if (!defined($error) and $res != 0) {
						if ($res & 127) {
							$error = 'Error executing run_tests.pl (terminated with signal '.($res & 127).(($res & 128) ? ', with coredump' : '').')!';
						}
						else {
							$error = 'Unknown error: run_tests.pl exited with value '.($res >> 8).'!';
						}
					}
					if (defined($error)) {
						$error = esc_url($error);
						print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
finished = 1;
document.cookie="last_error_text=$error; path=/";
location.replace("$conf_page");
// ]]>
</script>
DATA
					}
					else {
						$result_dir = esc_url($result_dir);
						print <<DATA;
<script language="javascript" type="text/javascript">
// <![CDATA[
finished = 1;
location.replace("tests_results.pl?details=$result_dir");
// ]]>
</script>
DATA
					}
					print "</body>\n</html>\n";
					exit;
				}
			}
			elsif (!$_GET{'goto'}) {
				$error = 'Script action not specified!';
			}
		}
		else {
			$error = "Cannot open file $session_profile for writing!<br />$!";
		}
	}
	else {
		# Another non-reachable branch (the situation should have been handled much earlier).
		$error = 'Session profile is not specified!';
	}
}

print 'HTTP/1.0 302 Moved' . CRLF;
if (defined($error)) {
	print 'Set-Cookie: last_error_text=' . esc_url($error) . '; path=/;' . CRLF;
}
print 'Location: ' . $conf_page . CRLF . CRLF;
